@@ -16,12 +16,14 @@ import os
 import re
 import shutil
 import sys
+import configparser
 from typing import List
 
 from flask import request
 from jinja2 import Environment, TemplateSyntaxError, FileSystemLoader, filters
 from werkzeug.urls import url_quote, url_encode
 
+import py3seed.ext
 from py3seed import registered_models, BaseModel, TemplateError, LayoutError
 from py3seed.utils import work_in, generate_names, get_layout_fields, parse_layout
 from py3seed.merge3 import Merge3
@@ -30,7 +32,7 @@ logger = logging.getLogger('pyseed')
 INCLUDES_FOLDER = '__includes'
 
 
-def _prepare_jinja2_env():
+def _prepare_jinja2_env(properties):
     """ Prepare env for rendering jinja2 templates. """
     #
     # For more env setting, please refer to https://jinja.palletsprojects.com/en/3.0.x/api/#jinja2.Environment
@@ -39,9 +41,9 @@ def _prepare_jinja2_env():
     # - keep_trailing_newline=True, Preserve the trailing newline when rendering templates
     #
     # trim_blocks=True and lstrip_blocks=True -> make sure lines of {% ... %} and {# ... #} will be removed completely in render result
-    # keep_trailing_newline=True -> keep trailing newline to so that you can use indent filter to include the macro with a tailing newline
     #
-    # For extension, plrease refer to https://jinja.palletsprojects.com/en/3.0.x/extensions/#loopcontrols-extension
+    # For extension, plrease refer to https://jinja.palletsprojects.com/en/3.0.x/extensions/
+    # - jinja2.ext.loopcontrols, add break and continue support in for loop
     #
     env = Environment(trim_blocks=True, lstrip_blocks=True, keep_trailing_newline=True, extensions=['jinja2.ext.loopcontrols'])
 
@@ -66,9 +68,15 @@ def _prepare_jinja2_env():
         fp = os.path.join(env.loader.searchpath[0], path)
         return os.path.exists(fp)
 
+    def getpro(key):
+        """ Get property value by key, from .pyseed-properties. """
+        return properties.get(key)
+
     env.globals['update_query'] = update_query
     env.globals['new_model'] = new_model
     env.globals['exists'] = exists
+    env.globals['getpro'] = getpro
+    env.add_extension(py3seed.ext.InlineGetpro)
 
     def split(value, separator):
         """ Split a string. """
@@ -79,8 +87,11 @@ def _prepare_jinja2_env():
         return value.items()
 
     def keys(value):
-        """ Return keys of a dict. """
-        return value.keys()
+        """ Return keys of dict. """
+        if type(value) is list:  # list of tuple
+            return map(lambda x: x[0], value)
+        else:
+            return value.keys()
 
     def quote(value):
         """ Add single quote to value if it is str, else return its __str__. """
@@ -114,6 +125,9 @@ def _prepare_jinja2_env():
         e.g,
         - values|match('name|title|\\w+_name')
         """
+        if not values:
+            return None
+        #
         matcher = re.compile(matcher if matcher.startswith('(') else f'({matcher})')
         if isinstance(values, dict):
             values = values.keys()
@@ -124,12 +138,15 @@ def _prepare_jinja2_env():
         # If no matching, return nothing
         return None
 
-    def fields(layout_or_schema, matcher=None):
+    def fields(layout_or_schema, matcher=None, **kwargs):
         """ Get the matched fields from layout or schema. """
+        if not layout_or_schema:
+            return []
+        #
         if isinstance(layout_or_schema, list):  # layout is [[{}, ...], ...]
-            _fields = list(get_layout_fields(layout_or_schema))
+            _fields = list(get_layout_fields(layout_or_schema, **kwargs))
         elif isinstance(layout_or_schema, dict):  # schema is dict {type: 'object', properties: {name, type, ... }}
-            _fields = list(layout_or_schema['properties'].keys())
+            _fields = list(layout_or_schema['columns'])
         else:
             raise ValueError(f'Unsupported type to calculate fields: {type(layout_or_schema)}')
         #
@@ -176,12 +193,13 @@ def _gen(ds: str = None):
     # 2. Load each models' schema and views, filtering models and views that needs to be generated
     #
     model_settings = {}
-    domain_names, view_names = set(), set()
+    domain_names, blueprint_view_names = set(), set()
     module_name = 'models'
     module_path = os.path.join('core', module_name, '__init__.py')
     if not os.path.exists(module_path):
         logger.error(f'Package does NOT exist at {module_path}')
         return False
+    #
     module_spec = importlib.util.spec_from_file_location(module_name, module_path)
     module = importlib.util.module_from_spec(module_spec)
     sys.modules[module_name] = module
@@ -191,6 +209,10 @@ def _gen(ds: str = None):
         attribute = getattr(module, attr)
         if inspect.isclass(attribute) and issubclass(attribute, BaseModel):
             model_name, model_class = attribute.__name__, attribute
+            # Model may be imported twice, in case of typo or being used as other name
+            if model_name in model_settings:
+                continue
+            #
             logger.info(f'{model_name}')
             # Parse model schema and views
             schema = model_class.schema()
@@ -225,13 +247,14 @@ def _gen(ds: str = None):
                     blueprint = 'public'
                 #
                 layout = v.strip()
-                logger.info(f'- {blueprint}/{name}: {layout}')
+                logger.info(f'- {"|".join(domains)}://{blueprint}/{name}: {layout}')
                 # Validate to make sure view name is unique
-                if name in view_names:
-                    logger.error(f'View name {v["name"]} is not unique')
+                blueprint_view_name = f'{blueprint}/{name}'
+                if blueprint_view_name in blueprint_view_names:
+                    logger.error(f'View {blueprint_view_name} is not unique')
                     return False
                 #
-                view_names.add(name)
+                blueprint_view_names.add(blueprint_view_name)
                 #
                 l = parse_layout(layout, schema)
                 views.append({
@@ -244,9 +267,6 @@ def _gen(ds: str = None):
                     'layout': layout,  # NOTE: layout stores the original layout string, parsed layout is stored in rows
                     **generate_names(name)
                 })
-            # Views may be empty if no views match
-            if not views:
-                continue
             #
             model_setting['views'] = views
             model_settings[model_name] = model_setting
@@ -255,6 +275,16 @@ def _gen(ds: str = None):
         logger.error('Can not find any models to gen')
         return False
     #
+    # Load properties file, which is ini format
+    # https://docs.python.org/3/library/configparser.html
+    #
+    properties = {}
+    properties_file = configparser.ConfigParser()
+    properties_file.read('.pyseed-properties')
+    for section in properties_file.sections():
+        for k in properties_file[section]:
+            properties[k] = properties_file[section][k]
+    #
     # For each domain:
     # 1. Build context
     # 2. Render jinja2 templates
@@ -306,22 +336,41 @@ def _gen(ds: str = None):
         for bp in blueprints:  # Blueprints
             bp_name = bp['name']
             logger.info(f'{bp_name}/')
+            models_by_name = {}
             for v in bp['views']:  # Views
                 v_name = v['name']
                 logger.info(f'  {v_name}')
+                #
+                v_model = v['model']
+                models_by_name[v_model['name']] = v_model
+                # Add relation models
+                for relation in v_model['schema']['relations']:
+                    models_by_name[relation] = model_settings[relation]
+            #
+            bp['models'] = list(models_by_name.values())
         # models & blueprints can be used in all templates
         context = {
+            'domain': domain,
             'models': model_settings,
             'blueprints': blueprints,
+            'warnings': [],  # return warnings to user
         }
         #
         # Do generation logic for each includes
         #
-        env = _prepare_jinja2_env()
+        env = _prepare_jinja2_env(properties)
         for include in includes:
             base = os.path.dirname(include)
             name = os.path.basename(include)
             _recursive_render(base, base, name, context, env)
+        #
+        # Print warnings of current domain
+        # TODO: Show more generation stats
+        #
+        if context['warnings']:
+            logger.warning(f'Encounter blew warnings when processing {domain}:')
+            for i, w in enumerate(context['warnings']):
+                logger.warning(f'{i + 1}: {w}')
 
 
 def _recursive_render(t_base, o_base, name, context, env):
@@ -400,7 +449,7 @@ def _recursive_render(t_base, o_base, name, context, env):
             #     www/templates/dash
             #     www/templates/demo
             #     ...
-            logger.info(f'Render {o_name}')
+            logger.info(f'Render {o_name}/')
             o_path = os.path.join(o_base, o_name)
             if not os.path.exists(o_path):
                 os.mkdir(o_path)
@@ -468,24 +517,30 @@ def _recursive_render(t_base, o_base, name, context, env):
                 # user-profile.html.0 exists, always generate code to user-profile.html.0, then you need to merge manually
                 #
                 # - Output name with additional suffix .1:
-                # Using .1(BASE), .11(THIS) and .111(OTHER) to perform a 3-way merge
+                # Using BASE, THIS and OTHER to perform a 3-way merge
                 # e.g,
-                # user-profile.html.1 exists, coping user-profile.html as user-profile.html.11, generating user-profile.html.111, then perform 3-way merge to user-profile.html
+                # user-profile.html.1 exists, coping user-profile.html.1 to user-profile.html.BASE and user-profile.html to user-profile.html.THIS, generating user-profile.html.OTHER, then perform 3-way merge to user-profile.html
                 #
                 o_file_0 = o_name.replace('.jinja2', '.0')
                 o_file_1 = o_name.replace('.jinja2', '.1')
-                o_file_11 = o_name.replace('.jinja2', '.11')
-                o_file_111 = o_name.replace('.jinja2', '.111')
+                o_file_base = o_name.replace('.jinja2', '.BASE')
+                o_file_this = o_name.replace('.jinja2', '.THIS')
+                o_file_other = o_name.replace('.jinja2', '.OTHER')
                 if os.path.exists(o_file_0):
                     o_file = o_file_0
                 elif os.path.exists(o_file_1):
-                    if os.path.exists(o_file_111):
-                        logger.warning(f'Please solve last merging conflicts of {o_file_raw}')
+                    # After manually merge, need to remove BASE/THIS/OTHER files mannually
+                    if os.path.exists(o_file_other):
+                        msg = f'Please solve last merging conflicts of {o_file_raw}'
+                        context['warnings'] += [msg]
+                        logger.warning(msg)
                         continue
+                    # BASE, copy from .1
+                    shutil.copyfile(o_file_1, o_file_base)
                     # THIS, copy from exsiting file
-                    shutil.copyfile(o_file_raw, o_file_11)
+                    shutil.copyfile(o_file_raw, o_file_this)
                     # OTHER, newly genearted file
-                    o_file = o_file_111
+                    o_file = o_file_other
                 else:
                     o_file = o_file_raw
                 #
@@ -511,13 +566,13 @@ def _recursive_render(t_base, o_base, name, context, env):
                 if os.path.exists(o_file_1):
                     logger.info(f'Perform 3-way merge of {o_file_raw}')
                     # BASE
-                    with open(o_file_1, 'r', encoding='utf-8') as f:
+                    with open(o_file_base, 'r', encoding='utf-8') as f:
                         base = f.read().splitlines(True)
                     # THIS
-                    with open(o_file_11, 'r', encoding='utf-8') as f:
+                    with open(o_file_this, 'r', encoding='utf-8') as f:
                         this = f.read().splitlines(True)
                     # OTHER
-                    with open(o_file_111, 'r', encoding='utf-8') as f:
+                    with open(o_file_other, 'r', encoding='utf-8') as f:
                         other = f.read().splitlines(True)
                     #
                     m3 = Merge3(base, other, this)
@@ -525,14 +580,18 @@ def _recursive_render(t_base, o_base, name, context, env):
                     # print('\n'.join(m3.merge_annotated()))
                     with open(o_file_raw, 'w', encoding='utf-8') as f:
                         f.write(merged)
+                    # copy OTHER to .1, so that next time we can use it as BASE
+                    shutil.copyfile(o_file_other, o_file_1)
                     # Has conficts, need to solve manually
                     if '=======' in merged:
-                        logger.warning(f'Please solve merging conflicts of {o_file_raw}')
+                        msg = f'Please solve merging conflicts of {o_file_raw}'
+                        context['warnings'] += [msg]
+                        logger.warning(msg)
                     else:
-                        # Rename .111 to .1 for next merging
-                        os.rename(o_file_111, o_file_1)
-                        # Remove .11
-                        os.remove(o_file_11)
+                        # Remove BASE/THIS/OTHER files
+                        os.remove(o_file_base)
+                        os.remove(o_file_this)
+                        os.remove(o_file_other)
                 # Remove out_key from context
                 if out_key:
                     del context[out_key]
