@@ -106,9 +106,10 @@ class SimpleEnum(object, metaclass=SimpleEnumMeta):
 
 
 class Format(SimpleEnum):
-    """ Predefined available formats, which should be used to control ui or api generation.
+    """ Built-in formats, which should be used to control ui or api generation.
 
-    NOTE: Name of format can NOT end with interger, as #FormatInteger has special meaning in view layout, e.g, '#summary4' means with summary format and 4 span
+    NOTE: Name of format can NOT end with interger, as #FormatInteger has special meaning in view layout:
+    e.g, '#summary4' means with summary format and 4 span, if you defined a format named 'gallery1' and want to use it view layout, the parsing logic will use 1 as the span and gallery as format.
     """
     # Below values are the same with OAS 3.0
     # - https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.0.3.md#data-types
@@ -128,6 +129,8 @@ class Format(SimpleEnum):
     RTE = 'rte'
     MARKDOWN = 'markdown'
     IMAGE = 'image'  # Image upload support
+    CAROUSEL = 'carousel'  # Image upload support and display in carousel
+    CANVAS = 'canvas'  # Image editing support
     AVATAR = 'avatar'  # User avatar
     FILE = 'file'  # File upload support
     VIDEO = 'video'  # Video upload support
@@ -410,15 +413,18 @@ class ModelMeta(ABCMeta):
         slots: Set[str] = namespace.get('__slots__', ())
         slots = {slots} if isinstance(slots, str) else set(slots)
         # Using for self-referencing, so we that we can remove all ForwardRef just after the class is created
-        # If use ForwardRef('other_class_name'), we do not have appropriate timing to replace it to its real class
-        # as we do not know if it is created or not
+        # Can not use to refernce other class, if using ForwardRef('other_class_name'), we do not have appropriate timing to replace it to its real class as we do not know if it is created or not
         has_forward_refs = False
         #
         for base in reversed(bases):
             if issubclass(base, BaseModel):  # True if base is BaseModel or its subclass
                 # BaseModel do not have __fields__
                 if base is not BaseModel:
+                    # Inherit the fields and properties defined by the parent class
+                    # e.g, User is a subclass of MongoModel, the _id field defined in MongoModel must be accessible in User
                     fields.update(deepcopy(base.__fields__))
+                    # e.g, User is a subclass of MongoModel, the id property defined in MongoModel must be accessible in User
+                    properties.update(deepcopy(base.__properties__))
                 # Id field's type should be defined in MongoModel/CacheModel, so we need to fetch them from bases
                 if '__id_type__' not in namespace:
                     namespace['__id_type__'] = base.__id_type__
@@ -478,6 +484,7 @@ class ModelMeta(ABCMeta):
         # Validation
         skips = set()
         if namespace.get('__qualname__') != 'BaseModel':
+            # print('Class: ' + namespace.get('__qualname__'))
             #
             # Iterate each annotation(field)
             # Similar to typing.get_type_hints, https://docs.python.org/3/library/typing.html
@@ -569,6 +576,7 @@ class ModelMeta(ABCMeta):
             # Properties
             #
             for attr in namespace:
+                # print('Properties:', attr, type(namespace.get(attr)))
                 if isinstance(namespace.get(attr), property):
                     properties[attr] = namespace.get(attr)
         #
@@ -696,6 +704,8 @@ class BaseModel(metaclass=ModelMeta):
     __icon__ = None
     __title__ = None
     __description__ = None
+    # When doing some logic that list models, e.g, drawing a table of users, we can use these columns to draw by default.
+    __columns__ = []
     # Views for this model, should be a dict
     # i.e,
     # {
@@ -741,8 +751,8 @@ class BaseModel(metaclass=ModelMeta):
         # Validate against schema
         # print(f'Validate {cls.__name__} with {data}')
         for field_name, field_type in cls.__fields__.items():
-            # Skip recursive validation in relation value, just keep the relation value
-            # please note: in save logic, we need to skip relation field, e.g, in mongosupport, we use self.dict(include_relations=False)
+            # Skip recursive validation in relation value, just keep the relation value, your program may need this relation value for further processing
+            # NOTE: in save logic, we need to skip relation field, e.g, in mongosupport, we use self.dict(include_relations=False)
             if isinstance(field_type, RelationField):
                 relation_value = data.get(field_name, Undefined)
                 if relation_value is not Undefined:
@@ -779,8 +789,9 @@ class BaseModel(metaclass=ModelMeta):
                             if id_:
                                 if not isinstance(id_, relation_type.__id_type__):
                                     id_ = relation_type.__id_type__(id_)
-                                #
-                                update_value.append(id_)
+                                # Prevent dulplicate id in update_value
+                                if id_ not in update_value:
+                                    update_value.append(id_)
                     elif relation_value is not None:
                         relation_type = source_field.type
                         if isinstance(relation_value, dict):
@@ -1082,6 +1093,10 @@ class BaseModel(metaclass=ModelMeta):
     ):
         """ Access model recrusively. """
         for field_name, field_value in self.__dict__.items():
+            # __dict__ may contain some other dynamic attributes, skip non-defined fields
+            if field_name not in self.__class__.__fields__:
+                continue
+            #
             field = self.__class__.__fields__[field_name]
             if isinstance(field, RelationField):
                 if include_relations:
@@ -1137,7 +1152,7 @@ class BaseModel(metaclass=ModelMeta):
         return []
 
     @classmethod
-    def schema(cls, layouts={}):
+    def schema(cls):
         """ To json schema dict.
 
         NOTE:
@@ -1277,6 +1292,7 @@ class BaseModel(metaclass=ModelMeta):
                 obj = {
                     'type': 'object',
                     'properties': properties,
+                    'columns': type_.__columns__ if type_.__columns__ else [f for f in properties.keys() if f != type_.__id_name__],
                     'requires': requires,
                     'editable': True,
                     'py_type': type_.__name__,
@@ -1293,18 +1309,6 @@ class BaseModel(metaclass=ModelMeta):
                     'id_name': type_.__id_name__,
                     'id_type': type_.__id_type__.__name__ if type_.__id_type__ else None,
                 }
-                # Inject layouts into schema
-                if layouts:
-                    layout = layouts.get(type_.__name__, {})
-                    if not layout.get('columns'):
-                        layout['columns'] = obj['requires']
-                    if not layout.get('read'):
-                        # Each field is a row
-                        layout['read'] = [[{'name': f, 'params': {}}] for f in obj['properties'].keys()]
-                    if not layout.get('form'):
-                        layout['form'] = layout['read']
-                    #
-                    obj.update(layout)
                 #
                 return obj
             elif type_ is str:
