@@ -147,6 +147,14 @@ class MongoModel(BaseModel):
     # Mongo ID field
     _id: ObjectId = None
 
+    @property
+    def id(self):
+        """ Make sure we can use a unified approach to obtain id from different types of models.
+
+        NOTE: Setting id still need to use _id, e.g, user._id = ObjectId()
+        """
+        return self._id
+
     #
     #
     # Class level pymongo api
@@ -229,6 +237,7 @@ class MongoModel(BaseModel):
 
     @classmethod
     def find_one_and_update(cls, filter_or_id, update, *args, **kwargs):
+        """ Find and update a single model. """
         collection = cls.get_collection(**kwargs)
         if filter_or_id is None:
             return None
