@@ -89,10 +89,9 @@ def parse_layout(body, schema):
 
     Layout is multi-line string defined in a model view.
     e.g,
-    'demo/user-profile': {              # action name
-        'domains': ['www'],             # domains that this action is available
-        'layout': '''#!form?param=1     # first line is action line defining action type and params, i.e, query, read, read_by_key, form
-            $#4,           0#8
+    'www|miniapp://demo/user-profile': {    # domains can be separated by |, blueprint is demo and view is user-profile
+        'layout': '''#!form?param=1         # first line is action line defining action type and params, i.e, query, read, read_by_key, form
+            1#4,           2#8
               avatar         name
               name           phone
               status         intro
@@ -148,7 +147,7 @@ def parse_layout(body, schema):
     }
     """
 
-    def _parse_lines(level, _lines, _schema):
+    def _parse_lines(level, _lines, _schema, _action):
         """ Recursively parse lines. """
         leading = '- ' * (level + 1)  # 2 spaces for each level
         # logger.debug('Parse lines:\n' + '\n'.join(_lines))
@@ -164,10 +163,6 @@ def parse_layout(body, schema):
                 indexes.append(index)
             elif indent < first_indent:
                 raise LayoutError(f'Invalid indent {indent} < {first_indent}: ' + line.replace(" ", "."))
-            else:
-                diff = first_indent - indent
-                if diff % 2 != 0:
-                    raise LayoutError(f'Invalid indent {indent} - {first_indent} is odd: ' + line.replace(" ", "."))
         # Parse each segment
         for i in range(0, len(indexes)):
             index = indexes[i]
@@ -205,8 +200,13 @@ def parse_layout(body, schema):
                 column = columns[j]
                 col_name = column['name']
                 #
-                col_lines = column.get('lines', None)
+                col_lines = column.get('lines', [])
                 logger.debug(f'{leading}Column ({i},{j}): {col_name}' + (('\n' + '\n'.join(col_lines)) if col_lines else ''))
+                # Validate each line's indent
+                for l in col_lines:
+                    indent = len(l) - len(l.lstrip())
+                    if indent % 2 != 0:
+                        raise LayoutError(f'Invalid indent {indent}: ' + l.replace(" ", "."))
                 #
                 # Column name possible values:
                 # 1) blank column prints only a placeholder
@@ -219,13 +219,18 @@ def parse_layout(body, schema):
                 # Hyphen column
                 elif col_name == '-':
                     pass
-                # Group column
+                # Group column, only contains number and dot(.)
                 elif col_name.replace('.', '').isdigit():
                     if not col_lines:
                         raise LayoutError(f'Group {col_name} should have inner layout')
                     #
-                    column['rows'] = _parse_lines(level + 1, col_lines, _schema)
+                    column['rows'] = _parse_lines(level + 1, col_lines, _schema, _action)
                 else:
+                    # May need other validations:
+                    # - update a back relation field
+                    # - update a non-editable field
+                    # However, we do NOT do above validation here, as it depends on formats also
+                    # Some formats are used for display only, i.e, summary, so we do not need to validate the fields under a summary group
                     if col_name not in _schema['properties']:  # type of _schema is always a object
                         raise LayoutError(f'Field {col_name} not found in schema')
                     #
@@ -240,7 +245,7 @@ def parse_layout(body, schema):
                     if inner_schema:
                         # Inner layout is optional for inner object
                         if col_lines:
-                            column['rows'] = _parse_lines(level + 1, col_lines, inner_schema)  # Schema passing recursively should always be object
+                            column['rows'] = _parse_lines(level + 1, col_lines, inner_schema, _action)  # Schema passing recursively should always be an object
                     else:
                         if col_lines:
                             raise LayoutError(f'{column_type.capitalize()} field {col_name} can not have inner layout')
@@ -322,7 +327,7 @@ def parse_layout(body, schema):
         if '?' in action_str:
             action, params = _parse_query_str(action_str)
     #
-    rows = _parse_lines(0, lines, schema)
+    rows = _parse_lines(0, lines, schema, action)
     #
     return {
         'action': action,
@@ -331,7 +336,7 @@ def parse_layout(body, schema):
     }
 
 
-def get_layout_fields(layout):
+def get_layout_fields(layout, exclude_formats=(Format.SUMMARY,)):
     """ Do not recursively parse inner object/array, only return current level field names. """
     if not layout:
         return []
@@ -343,7 +348,7 @@ def get_layout_fields(layout):
             # e.g,
             # - 1#summary4 is used to display a summary card of current object
             # - project#summary4 is used to display a summary card of a related object or inner object
-            if col['format'] == Format.SUMMARY:
+            if col['format'] in exclude_formats:
                 continue
             # Blank column
             if not col_name:
